/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#ifndef GRAPHIC_CONNECTOR_H
#define GRAPHIC_CONNECTOR_H

#include "graphic.h"

#include <OS/list.h>

class FlexibleConnector;
class GlyphViewer;

class Connector: public Graphic
{
	public:
		virtual ~Connector();

		virtual void connect(Connector *) = 0;
		virtual void disconnect(GlyphIndex) = 0;
		virtual void disconnect_all() = 0;
		virtual GlyphIndex c_count() const = 0;
		virtual Connector *connector(GlyphIndex) const = 0;

		virtual void disconnect(Connector *) = 0;

		virtual ClassId get_classid() const;
		virtual boolean is_a(ClassId) const;
	protected:
		Connector(Graphic * =nil);
};

declarePtrList(FlexConnList,FlexibleConnector)

class FixedConnector: public Connector
{
	public:
		FixedConnector(const Brush *,const Color *,const Color *,Transformer * =nil);
		virtual ~FixedConnector();

		virtual void connect(Connector *);
		virtual void disconnect(GlyphIndex);
		virtual void disconnect_all();
		virtual GlyphIndex c_count() const;
		virtual Connector *connector(GlyphIndex) const;

		virtual Glyph *clone() const;

		virtual void update(GlyphViewer * = nil);

		virtual ClassId get_classid() const;
		virtual boolean is_a(ClassId) const;
	protected:
		FixedConnector(Graphic * =nil);

		virtual void disconnect(Connector *);
	private:
		FlexConnList *_flex_list;
};

class FlexibleConnector: public Connector
{
	public:
		FlexibleConnector(const Brush *,const Color *,const Color *,Transformer * =nil);
		virtual ~FlexibleConnector();

		virtual void connect(Connector *);
		virtual void disconnect(GlyphIndex);
		virtual void disconnect_all();
		virtual GlyphIndex c_count() const;
		virtual Connector *connector(GlyphIndex) const;

		virtual Glyph *clone() const;

		virtual void update(GlyphViewer * = nil);

		virtual ClassId get_classid() const;
		virtual boolean is_a(ClassId) const;
	protected:
		FlexibleConnector(Graphic * = nil);

		virtual void disconnect(Connector *);
	private:
		FixedConnector *_fixed;
};

#endif
