/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#ifndef GRAPHIC_GLYPHVIEWER_H
#define GRAPHIC_GLYPHVIEWER_H

#include <InterViews/adjust.h>
#include <InterViews/event.h>
#include <InterViews/input.h>
#include <InterViews/observe.h>
#include <InterViews/transformer.h>

class Cursor;
class IOHandler;
class GlyphGrabber;
class GraphicMaster;
class GraphicZoomer;
class Slider;
class Tool;

class GlyphViewer: public InputHandler, public Adjustable
{
	public:
		GlyphViewer(Coord =-1.0,Coord =-1.0,Coord pagew=1600,Coord pageh=1600);
		virtual ~GlyphViewer();
	
		enum {left,middle,right};

		virtual void press(const Event &);
		virtual void drag(const Event &);
		virtual void release(const Event &);

		virtual void request(Requisition &) const;
		virtual void allocate(Canvas *,const Allocation &,Extension &);
		virtual void draw(Canvas *,const Allocation &) const;
		virtual void pick(Canvas *,const Allocation &,int depth,Hit &);

		virtual GraphicMaster *root() const;
		virtual void root(GraphicMaster *);

		virtual Tool *tool(unsigned =GlyphViewer::left) const;
		virtual void tool(Tool *,unsigned =GlyphViewer::left);

		void damage(Coord,Coord,Coord,Coord);
		void damage_all();
		void repair();
		void clip_all() const;
		Canvas *canvas() {return _canvas;}

		virtual void begin_adjustment(DimensionName);
		virtual void commit_adjustment(DimensionName);

		virtual Coord lower(DimensionName) const;
		virtual Coord upper(DimensionName) const;
		virtual Coord length(DimensionName) const;
		virtual Coord cur_lower(DimensionName) const;
		virtual Coord cur_upper(DimensionName) const;
		virtual Coord cur_length(DimensionName) const;
		virtual void scroll_to(DimensionName,Coord);

		virtual void scroll_forward(DimensionName);
		virtual void scroll_backward(DimensionName);
		virtual void page_forward(DimensionName);
		virtual void page_backward(DimensionName);

		void panner(Slider *s);
		Slider *panner() const;

		Adjustable *zoomer() const;
	protected:
		void initshape();
		void initgraphic();
		void inittimer();

		void do_scroll();
	private:
		float _width,_height;
		Allocation _a;

		Canvas *_canvas;
		Cursor *_cursor;
		long _delay;
		IOHandler *_timer;
		GraphicMaster *_root;
		GlyphGrabber *_grabber;
		Tool *_tool[GlyphViewer::right+1];
		GraphicZoomer *_zoomer;
		Coord _pagew,_pageh;
		Coord _curl,_curr;
		Coord _curb,_curt;
		Slider *_panner;
		void tick(long,long);
};

inline Adjustable *GlyphViewer::zoomer() const
{
	return (Adjustable *) _zoomer;
}

#endif
