/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/graphicmaster.h>
#include <Graphic/glyphviewer.h>
#include <Graphic/handles.h>
#include <Graphic/slrect.h>
#include <Graphic/conntool.h>
#include <Graphic/globals.h>

#include <IV-look/kit.h>

#include <InterViews/canvas.h>
#include <InterViews/display.h>
#include <InterViews/event.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>

//
// ConnectTool -- for connecting two graphics with a line
//

ConnectTool::ConnectTool(GraphicMaster *gm): Tool(gm)
{
	_g1=_g2=nil;
	_sll=nil;
}

ConnectTool::~ConnectTool()
{
}

boolean ConnectTool::grasp(const Event &e)
{
	float tol=2.0;
	GraphicMaster *gm=graphic_master();
	Transformer *tf=gm->transformer();
	Canvas *c=e.window()->canvas();
	BoxObj box(e.pointer_x()-tol, e.pointer_y()-tol, e.pointer_x()+tol, e.pointer_y()+tol);
	Graphic *target=gm->last_intersecting(box);

	if(target!=nil)
	{
		gm->clear_selection();
		_g1=target;
		_fromx=_tox=e.pointer_x();
		_fromy=_toy=e.pointer_y();
		_sll=new SlidingLine(_tox,_toy,_tox,_toy,c);
		return true;
	}
	return false;
}

boolean ConnectTool::manipulating(const Event &e)
{
	Coord x,y;
	GraphicMaster *gm=graphic_master();

	if(e.type()==Event::up)
		return false;
	if(e.type()==Event::motion)
	{
		x=e.pointer_x();
		y=e.pointer_y();
		_sll->translate(x-_tox,y-_toy);
		_tox=x;
		_toy=y;
	}
	return true;
}

boolean ConnectTool::effect(const Event &e)
{
	float tol=2.0;
	boolean flag=false;

	if(_sll!=nil)
	{
		Coord l,b,r,t;
		GraphicMaster *gm=graphic_master();
		Graphic *target;

		_tox=e.pointer_x();
		_toy=e.pointer_y();
		_sll->endpoints(l,b,r,t);
		BoxObj box(r-tol,t-tol,r+tol,t+tol);
		target=gm->last_intersecting(box);
		flag=(_g2=target)!=nil;
		delete _sll;
		_sll=nil;
	}
	return flag;
}

ClassId ConnectTool::get_classid() const
{
	return CONNECT_TOOL;
}

boolean ConnectTool::is_a(ClassId id) const
{
	return id==CONNECT_TOOL||Tool::is_a(id);
}
