/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/graphicmaster.h>
#include <Graphic/glyphviewer.h>
#include <Graphic/handles.h>
#include <Graphic/slrect.h>
#include <Graphic/graphictool.h>
#include <Graphic/globals.h>

#include <IV-look/kit.h>

#include <InterViews/canvas.h>
#include <InterViews/display.h>
#include <InterViews/event.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>

//
// GraphicTool -- inserts a predefined graphic
//

GraphicTool::GraphicTool(GraphicMaster *gm,Graphic *g): Tool(gm)
{
	_prototype=g;
	_slr=nil;
}

GraphicTool::~GraphicTool()
{
	delete _prototype;
}

boolean GraphicTool::grasp(const Event &e)
{
	Coord l,b,r,t,x,y;
	GraphicMaster *gm=graphic_master();
	Transformer *tf=gm->transformer();
	Canvas *c=e.window()->canvas();

	gm->clear_selection();
	_relx=e.pointer_x();
	_rely=e.pointer_y();
	prototype()->getbounds(l,b,r,t);
	tf->transform(l,b);
	tf->transform(r,t);
	prototype()->getcenter(x,y);
	tf->transform(x,y);
	_slr=new SlidingRect(l,b,r,t,c);
	_slr->translate(_relx-x,_rely-y);
	return true;
}

boolean GraphicTool::manipulating(const Event &e)
{
	Coord x,y;

	if(e.type()==Event::up)
		return false;
	if(e.type()==Event::motion)
	{
		x=e.pointer_x();
		y=e.pointer_y();
		_slr->translate(x-_relx,y-_rely);
		_relx=x;
		_rely=y;
	}
	return true;
}

boolean GraphicTool::effect(const Event &e)
{
	if(_slr!=nil)
	{
		GraphicMaster *gm=graphic_master();
		Transformer *tf=erase_slrect();
		Transformer *tm=gm->transformer();
		Graphic *g=(Graphic *) prototype()->clone();
		Coord l,b,r,t;
		GlyphViewer *gv=gm->glyphviewer();

		graphic_master()->append_(g);
		tf->transform(0,0,l,b);
		tm->inverse_transform(l,b);
		tm->inverse_transform(0,0,r,t);
		g->translate(l-r,b-t);
		Resource::unref(tf);
		gm->select(g,e.window()->canvas());
		if(gv!=nil)
		{
			float tol=Handles::tol();

			g->getbounds(l,b,r,t);
			gv->damage(l-tol,b-tol,r+tol,t+tol);
			gv->repair();
		}
		return true;
	}
	return false;
}

//

Transformer *GraphicTool::erase_slrect()
{
	Transformer *t=_slr->transformer();

	Resource::ref(t);
	delete _slr;
	_slr=nil;
	return t;
}

ClassId GraphicTool::get_classid() const
{
	return GRAPHIC_TOOL;
}

boolean GraphicTool::is_a(ClassId id) const
{
	return id==GRAPHIC_TOOL||Tool::is_a(id);
}
