/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/movecmd.h>
#include <Graphic/graphicmaster.h>
#include <Graphic/glyphviewer.h>
#include <Graphic/handles.h>

//
// MoveCmd - a command for translating the selection of the GraphicMaster
//

MoveCmd::MoveCmd(GraphicMaster *gm,Coord x,Coord y): Command(gm)
{
	_dx=x;
	_dy=y;
}

void MoveCmd::execute()
{
	GraphicMaster *gm=graphic_master();
	GlyphViewer *gv=gm->glyphviewer();
	GraphicList *s=gm->selection();
	long i,count=s->count();
	Coord tol=Handles::tol();

	for(i=0;i<count;i++)
	{
		Coord l,b,r,t;
		Graphic *g=s->item(i);

		g->getbounds(l,b,r,t);                 // erasing the graphic
		gv->damage(l-tol,b-tol,r+tol,t+tol);   // at the old position
		g->translate(_dx,_dy);                 // translating
		g->getbounds(l,b,r,t);                 // drawing the graphic
		gv->damage(l-tol,b-tol,r+tol,t+tol);   // at the new position
	}
	gm->update(gv);                                // remember all connectors
}

ClassId MoveCmd::get_classid() const
{
	return MOVE_CMD;
}

boolean MoveCmd::is_a(ClassId id) const
{
	return id==MOVE_CMD || Command::is_a(id);
}

