/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/graphicmaster.h>
#include <Graphic/glyphviewer.h>
#include <Graphic/handles.h>
#include <Graphic/slrect.h>
#include <Graphic/selecttool.h>
#include <Graphic/globals.h>

#include <IV-look/kit.h>

#include <InterViews/canvas.h>
#include <InterViews/display.h>
#include <InterViews/event.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>

//
// SelectTool -- selecting and moving graphics with mouse pointer
//

SelectTool::SelectTool(GraphicMaster *gm): Tool(gm)
{
	_slr=nil;
	_rx=_ry=0;
}

SelectTool::~SelectTool()
{
	delete _slr;
}

boolean SelectTool::grasp(const Event &e)
{
	GraphicMaster *gm=graphic_master();
	Transformer *tf=gm->transformer();
	Canvas *c=e.window()->canvas();
	float tol=2.0;
	BoxObj box(e.pointer_x()-tol,e.pointer_y()-tol,e.pointer_x()+tol,e.pointer_y()+tol);
	Graphic *target=gm->last_intersecting(box);
	boolean flag1=e.shift_is_down();
	boolean flag2=gm->is_selected(target);

	if(!(flag1 || flag2))
		gm->clear_selection();
	_rx=e.pointer_x();
	_ry=e.pointer_y();
	if(target==nil)
	{
		_slr=new GrowingRect(_rx,_ry,_rx,_ry,e.window()->canvas());
		return true;
	}
	else
	{
		if(flag1)
		{
			gm->select(target,c);
			return false;
		}
		else
			if(!(flag1 || flag2))
				gm->select(target,c);
	}
	return true;
}


boolean SelectTool::manipulating(const Event &e)
{
	Coord x,y,l,b,r,t;
	Transformer *tf=graphic_master()->transformer();

	if(e.type()==Event::up)
		return false;
	if(e.type()==Event::motion)
	{
		if(_slr!=nil)
		{
			_rx=e.pointer_x();
			_ry=e.pointer_y();
			((GrowingRect *) _slr)->flex_point(_rx,_ry);
		}
	}
	return true;
}

boolean SelectTool::effect(const Event &e)
{
	if(_slr!=nil)
	{
		Transformer *tf=erase_slrect();
		Coord l,b,r,t,x,y;
		GraphicMaster *gm=graphic_master();
		GlyphIndex i,count=gm->count_();

		tf->transform(0,0,l,b);
		tf->transform(1,1,x,y);
		Resource::unref(tf);
		r=Math::max(l,x);
		t=Math::max(b,y);
		l=Math::min(l,x);
		b=Math::min(b,y);

		BoxObj box(l,b,r,t);

		for(i=0;i<count;i++)
		{
			Graphic *g=gm->component_(i);

			if(g->intersects(box))
				gm->select(g,e.window()->canvas(),true);
		}
		return true;
	}
	return false;
}

//

Transformer *SelectTool::erase_slrect()
{
	Transformer *t=_slr->transformer();

	Resource::ref(t);
	delete _slr;
	_slr=nil;
	return t;
}

ClassId SelectTool::get_classid() const
{
	return SELECT_TOOL;
}

boolean SelectTool::is_a(ClassId id) const
{
	return id==SELECT_TOOL||Tool::is_a(id);
}

