#include <Graphic/conntool.h>
#include <Graphic/globals.h>
#include <Graphic/glyphviewer.h>
#include <Graphic/graphicmaster.h>
#include <Graphic/handles.h>
#include <Graphic/lines.h>
#include <Graphic/link.h>
#include <Graphic/slrect.h>

#include "lineconnect.h"

#include <IV-look/kit.h>

#include <InterViews/canvas.h>
#include <InterViews/event.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>

#include <OS/list.h>
#include <OS/math.h>

LineConnectTool::LineConnectTool(GraphicMaster *gm,Link *g): ConnectTool(gm)
{
	_proto=g;
}

LineConnectTool::~LineConnectTool()
{
	delete _proto;
}

boolean LineConnectTool::effect(const Event &e)
{
	Graphic *g1,*g2;

	ConnectTool::effect(e);
	if(!connection(g1,g2))
		return false;
	g1=get_connector(g1);
	g2=get_connector(g2);
	if(g1!=nil && g2!=nil)
	{
		GraphicMaster *gm=graphic_master();
		GlyphViewer *gv=gm->glyphviewer();
		Graphic *li=_proto->get_line();

		g1=new Link((FixedConnector *) g1,(FixedConnector *) g2,li->brush(),li->stroke(),li->fill());
		gm->prepend_(g1);
		g1->update();
		if(gv!=nil)
		{
			Coord l,b,r,t;
			float tol=Handles::tol();

			g1->getbounds(l,b,r,t);
			gv->damage(l-tol,b-tol,r+tol,t+tol);
			gv->repair();
		}
		return true;
	}
	return false;
}

ClassId LineConnectTool::get_classid() const
{
	return LINE_CONNECT_TOOL;
}

boolean LineConnectTool::is_a(ClassId id) const
{
	return id==LINE_CONNECT_TOOL || ConnectTool::is_a(id);
}

//

Graphic *LineConnectTool::get_connector(Graphic *g)
{
	if(g->is_a(FIXEDCONN_GRAPHIC))
		return g;
	if(g->is_a(POLY_GRAPHIC))
	{
		Graphic *g1;
		GlyphIndex i,count=g->count_();

		for(i=0;i<count;i++)
		{
			g1=g->component_(i);
			if(g1->is_a(FIXEDCONN_GRAPHIC))
				return g1;
		}
	}
	return nil;
}
